@@ -1,49 +1,45 @@
 /*
- * rcracki_mt is a multithreaded implementation and fork of the original 
+ * freerainbowtables is a multithreaded implementation and fork of the original 
  * RainbowCrack
  *
  * Copyright (C) Zhu Shuanglei <shuanglei@hotmail.com>
  * Copyright Martin Westergaard Jørgensen <martinwj2005@gmail.com>
  * Copyright 2009, 2010 Daniël Niggebrugge <niggebrugge@fox-it.com>
- * Copyright 2009, 2010 James Nobis <frt@quelrod.net>
+ * Copyright 2009, 2010, 2011 James Nobis <quel@quelrod.net>
  * Copyright 2010 Yngve AAdlandsvik
+ * Copyright 2008, 2009, 2010, 2011 Steve Thomas (Sc00bz)
  *
- * This file is part of rcracki_mt.
+ * This file is part of freerainbowtables.
  *
- * rcracki_mt is free software: you can redistribute it and/or modify
+ * freerainbowtables is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * rcracki_mt is distributed in the hope that it will be useful,
+ * freerainbowtables is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with rcracki_mt.  If not, see <http://www.gnu.org/licenses/>.
+ * along with freerainbowtables.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#if defined(_WIN32) && !defined(__GNUC__)
-	#pragma warning(disable : 4786 4267 4018)
-#endif
-
 #include "ChainWalkContext.h"
 
 #include <ctype.h>
 
 //////////////////////////////////////////////////////////////////////
 
-string CChainWalkContext::m_sHashRoutineName;
+std::string CChainWalkContext::m_sHashRoutineName;
 HASHROUTINE CChainWalkContext::m_pHashRoutine;
 int CChainWalkContext::m_nHashLen;
+uint8 CChainWalkContext::RTfileFormatId;
 int CChainWalkContext::m_nPlainLenMinTotal = 0;
 int CChainWalkContext::m_nPlainLenMaxTotal = 0;
 int CChainWalkContext::m_nHybridCharset = 0;
-bool CChainWalkContext::isOldRtFormat = false;
-bool CChainWalkContext::isRti2RtFormat = false;
-vector<stCharset> CChainWalkContext::m_vCharset;
-uint64 CChainWalkContext::m_nPlainSpaceUpToX[MAX_PLAIN_LEN + 1];
+std::vector<stCharset> CChainWalkContext::m_vCharset;
+uint64 CChainWalkContext::m_nPlainSpaceUpToX[MAX_PLAIN_LEN];
 uint64 CChainWalkContext::m_nPlainSpaceTotal;
 unsigned char CChainWalkContext::m_Salt[MAX_SALT_LEN];
 int CChainWalkContext::m_nSaltLen = 0;
@@ -60,7 +56,7 @@ CChainWalkContext::~CChainWalkContext()
 {
 }
 
-bool CChainWalkContext::LoadCharset(string sName)
+bool CChainWalkContext::LoadCharset( std::string sName )
 {
 	m_vCharset.clear();
 	if (sName == "byte")
@@ -69,45 +65,55 @@ bool CChainWalkContext::LoadCharset(string sName)
 		int i;
 		for (i = 0x00; i <= 0xff; i++)
 			tCharset.m_PlainCharset[i] = (unsigned char) i;
-		tCharset.m_nPlainCharsetLen = 256;
+		tCharset.m_nPlainCharsetLen = MAX_PLAIN_LEN;
 		tCharset.m_sPlainCharsetName = sName;
 		tCharset.m_sPlainCharsetContent = "0x00, 0x01, ... 0xff";
 		m_vCharset.push_back(tCharset);
 		return true;
 	}
 	if(sName.substr(0, 6) == "hybrid") // Hybrid charset consisting of 2 charsets
-		m_nHybridCharset = 1;		
+	{
+		if (sName.substr(6, 2) == "2(" )
+			m_nHybridCharset = 2;
+		else
+			m_nHybridCharset = 1;		
+	}
 	else
 		m_nHybridCharset = 0;
 	
 	bool readCharset = false;
-	vector<string> vLine;
+	std::vector<std::string> vLine;
 
-	if ( ReadLinesFromFile("charset.txt", vLine) )
-		readCharset = true;
-	else if ( ReadLinesFromFile(GetApplicationPath() + "charset.txt", vLine) )
-		readCharset = true;
+	#ifdef BOINC
+		if ( boinc_ReadLinesFromFile( "charset.txt", vLine ) )
+			readCharset = true;
+	#else
+		if ( ReadLinesFromFile("charset.txt", vLine) )
+			readCharset = true;
+		else if ( ReadLinesFromFile(GetApplicationPath() + "charset.txt", vLine) )
+			readCharset = true;
+	#endif
 
-	if (readCharset)
+	if ( readCharset )
 	{
-		UINT4 i;
-		for (i = 0; i < vLine.size(); i++)
+		int i;
+		for (i = 0; (uint32)i < vLine.size(); i++)
 		{
 			// Filter comment
 			if (vLine[i][0] == '#')
 				continue;
 
-			vector<string> vPart;
+			std::vector<std::string> vPart;
 			if (SeperateString(vLine[i], "=", vPart))
 			{
 				// sCharsetName
-				string sCharsetName = TrimString(vPart[0]);
+				std::string sCharsetName = TrimString(vPart[0]);
 				if (sCharsetName == "")
 					continue;
 								
 				// sCharsetName charset check
 				bool fCharsetNameCheckPass = true;
-				UINT4 j;
+				uint32 j;
 				for (j = 0; j < sCharsetName.size(); j++)
 				{
 					if (   !isalpha(sCharsetName[j])
@@ -125,7 +131,7 @@ bool CChainWalkContext::LoadCharset(string sName)
 				}
 
 				// sCharsetContent
-				string sCharsetContent = TrimString(vPart[1]);
+				std::string sCharsetContent = TrimString(vPart[1]);
 				if (sCharsetContent == "" || sCharsetContent == "[]")
 					continue;
 				if (sCharsetContent[0] != '[' || sCharsetContent[sCharsetContent.size() - 1] != ']')
@@ -134,7 +140,7 @@ bool CChainWalkContext::LoadCharset(string sName)
 					continue;
 				}
 				sCharsetContent = sCharsetContent.substr(1, sCharsetContent.size() - 2);
-				if (sCharsetContent.size() > 256)
+				if (sCharsetContent.size() > MAX_PLAIN_LEN)
 				{
 					printf("charset content %s too long\n", sCharsetContent.c_str());
 					continue;
@@ -142,10 +148,10 @@ bool CChainWalkContext::LoadCharset(string sName)
 
 				//printf("%s = [%s]\n", sCharsetName.c_str(), sCharsetContent.c_str());
 
-				// Is it the wanted charset?
-				if(m_nHybridCharset == 1)
+				// Is it a hybrid?
+				if( m_nHybridCharset != 0 )
 				{
-					vector<tCharset> vCharsets;
+					std::vector<tCharset> vCharsets;
 					GetHybridCharsets(sName, vCharsets);
 					if(sCharsetName == vCharsets[m_vCharset.size()].sName)
 					{
@@ -159,7 +165,10 @@ bool CChainWalkContext::LoadCharset(string sName)
 						m_vCharset.push_back(tCharset);
 						if(vCharsets.size() == m_vCharset.size())
 							return true;
-						i = 0; // Start the lookup over again for the next charset
+						//i = 0; // Start the lookup over again for the next charset
+						// Sc00bz indicates this fixes a bug of skipping line 1
+						// of charset.txt
+						i = -1; // Start the lookup over again for the next charset
 					}						
 				}
 				else if (sCharsetName == sName)
@@ -178,12 +187,13 @@ bool CChainWalkContext::LoadCharset(string sName)
 	}
 	else
 		printf("can't open charset configuration file\n");
+
 	return false;
 }
 
 //////////////////////////////////////////////////////////////////////
 
-bool CChainWalkContext::SetHashRoutine(string sHashRoutineName)
+bool CChainWalkContext::SetHashRoutine( std::string sHashRoutineName )
 {
 	CHashRoutine hr;
 	hr.GetHashRoutine(sHashRoutineName, m_pHashRoutine, m_nHashLen);
@@ -196,7 +206,7 @@ bool CChainWalkContext::SetHashRoutine(string sHashRoutineName)
 		return false;
 }
 
-bool CChainWalkContext::SetPlainCharset(string sCharsetName, int nPlainLenMin, int nPlainLenMax)
+bool CChainWalkContext::SetPlainCharset( std::string sCharsetName, int nPlainLenMin, int nPlainLenMax)
 {
 	// m_PlainCharset, m_nPlainCharsetLen, m_sPlainCharsetName, m_sPlainCharsetContent
 	if (!LoadCharset(sCharsetName))
@@ -218,24 +228,42 @@ bool CChainWalkContext::SetPlainCharset(string sCharsetName, int nPlainLenMin, i
 	m_nPlainLenMaxTotal = 0;
 	m_nPlainLenMinTotal = 0;
 	uint64 nTemp = 1;
-	UINT4 j, k = 1;
+	uint32 j, k = 1;
+	int i = 1;
 	for(j = 0; j < m_vCharset.size(); j++)
 	{
-		int i;
 		m_nPlainLenMaxTotal += m_vCharset[j].m_nPlainLenMax;
 		m_nPlainLenMinTotal += m_vCharset[j].m_nPlainLenMin;
+		m_vCharset[j].m_nPlainSpaceUpToX[0] = 0;
+		uint64 nTemp2 = 1;
+			
+		// XXX assumes each sub keyspace starts at length 1
 		for (i = 1; i <= m_vCharset[j].m_nPlainLenMax; i++)
 		{			
 			nTemp *= m_vCharset[j].m_nPlainCharsetLen;
+			nTemp2 *= m_vCharset[j].m_nPlainCharsetLen;
+
 			if (i < m_vCharset[j].m_nPlainLenMin)
+			{
 				m_nPlainSpaceUpToX[k] = 0;
+				m_vCharset[j].m_nPlainSpaceUpToX[i] = 0;
+			}
 			else
+			{
 				m_nPlainSpaceUpToX[k] = m_nPlainSpaceUpToX[k - 1] + nTemp;
+				m_vCharset[j].m_nPlainSpaceUpToX[i] = m_vCharset[j].m_nPlainSpaceUpToX[i - 1] + nTemp2;
+			}
+
 			k++;
-		}		
+		}
+
+		m_vCharset[j].m_nPlainSpaceTotal = m_vCharset[j].m_nPlainSpaceUpToX[i-1];
+
+		if ( m_nPlainSpaceTotal == 0 )
+			m_nPlainSpaceTotal = m_vCharset[j].m_nPlainSpaceTotal;
+		else
+			m_nPlainSpaceTotal *= m_vCharset[j].m_nPlainSpaceTotal;
 	}
-	// m_nPlainSpaceTotal
-	m_nPlainSpaceTotal = m_nPlainSpaceUpToX[m_nPlainLenMaxTotal];
 
 	return true;
 }
@@ -259,16 +287,16 @@ bool CChainWalkContext::SetSalt(unsigned char *Salt, int nSaltLength)
 	return true;
 }
 
-bool CChainWalkContext::SetupWithPathName(string sPathName, int& nRainbowChainLen, int& nRainbowChainCount)
+bool CChainWalkContext::SetupWithPathName( std::string sPathName, int& nRainbowChainLen, int& nRainbowChainCount)
 {
 	// something like lm_alpha#1-7_0_100x16_test.rt
 
 #ifdef _WIN32
-	string::size_type nIndex = sPathName.find_last_of('\\');
+	std::string::size_type nIndex = sPathName.find_last_of('\\');
 #else
-	string::size_type nIndex = sPathName.find_last_of('/');
+	std::string::size_type nIndex = sPathName.find_last_of('/');
 #endif
-	if (nIndex != string::npos)
+	if (nIndex != std::string::npos)
 		sPathName = sPathName.substr(nIndex + 1);
 
 	if (sPathName.size() < 3)
@@ -278,15 +306,15 @@ bool CChainWalkContext::SetupWithPathName(string sPathName, int& nRainbowChainLe
 	}
 	if (sPathName.substr(sPathName.size() - 5) == ".rti2")
 	{
-		isRti2RtFormat = true;
+		RTfileFormatId = getRTfileFormatId( "RTI2" );
 	}
 	else if (sPathName.substr(sPathName.size() - 4) == ".rti")
 	{
-		isOldRtFormat = false;
+		RTfileFormatId = getRTfileFormatId( "RTI" );
 	}
 	else if (sPathName.substr(sPathName.size() - 3) == ".rt")
 	{
-		isOldRtFormat = true;
+		RTfileFormatId = getRTfileFormatId( "RT" );
 	}
 	else
 	{
@@ -295,21 +323,21 @@ bool CChainWalkContext::SetupWithPathName(string sPathName, int& nRainbowChainLe
 	}
 
 	// Parse
-	vector<string> vPart;
+	std::vector<std::string> vPart;
 	if (!SeperateString(sPathName, "___x_", vPart))
 	{
 		printf("filename %s not identified\n", sPathName.c_str());
 		return false;
 	}
 
-	string sHashRoutineName   = vPart[0];
+	std::string sHashRoutineName   = vPart[0];
 	int nRainbowTableIndex    = atoi(vPart[2].c_str());
 	nRainbowChainLen          = atoi(vPart[3].c_str());
 	nRainbowChainCount        = atoi(vPart[4].c_str());
 
 	// Parse charset definition
-	string sCharsetDefinition = vPart[1];
-	string sCharsetName;
+	std::string sCharsetDefinition = vPart[1];
+	std::string sCharsetName;
 	int nPlainLenMin = 0, nPlainLenMax = 0;		
 
 //	printf("Charset: %s", sCharsetDefinition.c_str());
@@ -320,7 +348,7 @@ bool CChainWalkContext::SetupWithPathName(string sPathName, int& nRainbowChainLe
 	}
 	else
 	{
-		if ( sCharsetDefinition.find('#') == string::npos )		// For backward compatibility, "#1-7" is implied
+		if ( sCharsetDefinition.find('#') == std::string::npos )		// For backward compatibility, "#1-7" is implied
 		{			
 			sCharsetName = sCharsetDefinition;
 			nPlainLenMin = 1;
@@ -328,7 +356,7 @@ bool CChainWalkContext::SetupWithPathName(string sPathName, int& nRainbowChainLe
 		}
 		else
 		{
-			vector<string> vCharsetDefinitionPart;
+			std::vector<std::string> vCharsetDefinitionPart;
 			if (!SeperateString(sCharsetDefinition, "#-", vCharsetDefinitionPart))
 			{
 				printf("filename %s not identified\n", sPathName.c_str());
@@ -359,7 +387,7 @@ bool CChainWalkContext::SetupWithPathName(string sPathName, int& nRainbowChainLe
 	return true;
 }
 
-string CChainWalkContext::GetHashRoutineName()
+std::string CChainWalkContext::GetHashRoutineName()
 {
 	return m_sHashRoutineName;
 }
@@ -369,12 +397,12 @@ int CChainWalkContext::GetHashLen()
 	return m_nHashLen;
 }
 
-string CChainWalkContext::GetPlainCharsetName()
+std::string CChainWalkContext::GetPlainCharsetName()
 {
 	return m_vCharset[0].m_sPlainCharsetName;
 }
 
-string CChainWalkContext::GetPlainCharsetContent()
+std::string CChainWalkContext::GetPlainCharsetContent()
 {
 	return m_vCharset[0].m_sPlainCharsetContent;
 }
@@ -404,24 +432,44 @@ void CChainWalkContext::Dump()
 	printf("hash routine: %s\n", m_sHashRoutineName.c_str());
 	printf("hash length: %d\n", m_nHashLen);
 
-	printf("plain charset: ");
-	unsigned int i;
-	for (i = 0; i < m_vCharset[0].m_nPlainCharsetLen; i++)
+	for ( uint32 i = 0; i < m_vCharset.size(); i++ )
 	{
-		if (isprint(m_vCharset[0].m_PlainCharset[i]))
-			printf("%c", m_vCharset[0].m_PlainCharset[i]);
-		else
-			printf("?");
+		printf( "m_vCharset[%d].m_nPlainCharSetLen: %d\n", i, m_vCharset[i].m_nPlainCharsetLen );
+
+		printf("plain charset: ");
+		
+		for ( uint32 j = 0; j < m_vCharset[i].m_nPlainCharsetLen; j++ )
+		{
+			if (isprint(m_vCharset[i].m_PlainCharset[j]))
+				printf("%c", m_vCharset[i].m_PlainCharset[j]);
+			else
+				printf("?");
+		}
+		printf("\n");
+
+		for ( int j = 0; j <= m_vCharset[i].m_nPlainLenMax; j++ )
+		{
+			printf( "m_vCharset[%d].m_nPlainSpaceUpToX[%d]: %llu\n"
+				, i, j, m_vCharset[i].m_nPlainSpaceUpToX[j] );
+		}
+		
+		printf("plain charset in hex: ");
+
+		for ( uint32 j = 0; j < m_vCharset[i].m_nPlainCharsetLen; j++ )
+			printf("%02x ", m_vCharset[i].m_PlainCharset[j]);
+		printf("\n");
+
+		printf("plain length range: %d - %d\n", m_vCharset[i].m_nPlainLenMin, m_vCharset[i].m_nPlainLenMax);
+		printf("plain charset name: %s\n", m_vCharset[i].m_sPlainCharsetName.c_str());
+		printf("plain subkey space total: %s\n", uint64tostr(m_vCharset[i].m_nPlainSpaceTotal).c_str());
+	}
+		
+	for ( int i = 0; i <= m_nPlainLenMaxTotal; i++ )
+	{
+		printf( "m_nPlainSpaceUpToX[%d]: %llu\n"
+			, i, m_nPlainSpaceUpToX[i] );
 	}
-	printf("\n");
 
-	printf("plain charset in hex: ");
-	for (i = 0; i < m_vCharset[0].m_nPlainCharsetLen; i++)
-		printf("%02x ", m_vCharset[0].m_PlainCharset[i]);
-	printf("\n");
-
-	printf("plain length range: %d - %d\n", m_vCharset[0].m_nPlainLenMin, m_vCharset[0].m_nPlainLenMax);
-	printf("plain charset name: %s\n", m_vCharset[0].m_sPlainCharsetName.c_str());
 	//printf("plain charset content: %s\n", m_sPlainCharsetContent.c_str());
 	//for (i = 0; i <= m_nPlainLenMax; i++)
 	//	printf("plain space up to %d: %s\n", i, uint64tostr(m_nPlainSpaceUpToX[i]).c_str());
@@ -442,11 +490,73 @@ void CChainWalkContext::SetHash(unsigned char* pHash)
 	memcpy(m_Hash, pHash, m_nHashLen);
 }
 
+int CChainWalkContext::normalIndexToPlain(uint64 index, uint64 *plainSpaceUpToX, unsigned char *charSet, int charSetLen, int min, int max, unsigned char *plain)
+{
+	int a;
+
+	for ( a = max - 1; a >= min; a-- )
+	{
+		if ( index >= plainSpaceUpToX[a])
+			break;
+	}
+
+	uint32 plainLen = a + 1;
+
+	index -= plainSpaceUpToX[a]; // plainLen - 1 == a
+
+	for ( a = plainLen - 1; a >= 0; a-- )
+	{
+		// XXX this is optimized for 32-bit platforms
+#if defined(_WIN32) && !defined(__GNUC__)
+		if (index < 0x100000000I64)
+			break;
+#else
+		if (index < 0x100000000llu)
+			break;
+#endif
+		plain[a] = charSet[index % charSetLen];
+		index /= charSetLen;
+	}
+
+	unsigned int index32 = (unsigned int) index;
+	for ( ; a >= 0; a-- )
+	{
+		// remarks from Sc00bz
+		// Note the lack of assembly code.
+		// Assembly code is not needed since all the variables are in the stack.
+		// If you add in assembly code it will be slower than the compiler's code.
+
+		plain[a] = charSet[index32 % charSetLen];
+		index32 /= charSetLen;
+	}
+
+	return plainLen;
+}
+
+void CChainWalkContext::IndexToPlain()
+{
+	m_nPlainLen = 0;
+	uint64 indexTmp = m_nIndex;
+
+	uint32 numKeySpaces = m_vCharset.size();
+
+	for ( uint32 a = 0; a < numKeySpaces - 1; a++ )
+	{
+		m_vCharset[a].m_nIndexX = indexTmp % m_vCharset[a].m_nPlainSpaceTotal;
+		indexTmp /= m_vCharset[a].m_nPlainSpaceTotal;
+		m_nPlainLen += normalIndexToPlain(m_vCharset[a].m_nIndexX, m_vCharset[a].m_nPlainSpaceUpToX, m_vCharset[a].m_PlainCharset, m_vCharset[a].m_nPlainCharsetLen, m_vCharset[a].m_nPlainLenMin, m_vCharset[a].m_nPlainLenMax, m_Plain + m_nPlainLen);
+	}
+
+	m_vCharset[numKeySpaces-1].m_nIndexX = indexTmp;
+	m_nPlainLen += normalIndexToPlain(m_vCharset[numKeySpaces-1].m_nIndexX, m_vCharset[numKeySpaces-1].m_nPlainSpaceUpToX, m_vCharset[numKeySpaces-1].m_PlainCharset, m_vCharset[numKeySpaces-1].m_nPlainCharsetLen, m_vCharset[numKeySpaces-1].m_nPlainLenMin, m_vCharset[numKeySpaces-1].m_nPlainLenMax, m_Plain + m_nPlainLen);
+}
+
+
+/*
 void CChainWalkContext::IndexToPlain()
 {
 	int i;
 	m_nPlainLen = 0;
-///*
 	for (i = m_nPlainLenMaxTotal - 1; i >= m_nPlainLenMinTotal - 1; i--)
 	{
 		if (m_nIndex >= m_nPlainSpaceUpToX[i])
@@ -455,16 +565,6 @@ void CChainWalkContext::IndexToPlain()
 			break;
 		}
 	}
-
-	// this is an optimized version of the above
-/*
-	for (i = m_nPlainLenMaxTotal - 1; i >= m_nPlainLenMinTotal - 1
-		&& m_nIndex < m_nPlainSpaceUpToX[i]; i--)
-	{ }
-	
-	m_nPlainLen = i + 1;
-*/
-
 	if(m_nPlainLen == 0)
 		m_nPlainLen = m_nPlainLenMinTotal;
 	uint64 nIndexOfX = m_nIndex - m_nPlainSpaceUpToX[m_nPlainLen - 1];
@@ -476,7 +576,7 @@ void CChainWalkContext::IndexToPlain()
 	for (i = m_nPlainLen - 1; i >= 0; i--)
 	{
 		int nCharsetLen = 0;
-		for(UINT4 j = 0; j < m_vCharset.size(); j++)
+		for(uint32 j = 0; j < m_vCharset.size(); j++)
 		{
 			nCharsetLen += m_vCharset[j].m_nPlainLenMax;
 			if(i < nCharsetLen) // We found the correct charset
@@ -503,7 +603,7 @@ void CChainWalkContext::IndexToPlain()
 #endif
 
 		int nCharsetLen = 0;
-		for(UINT4 j = 0; j < m_vCharset.size(); j++)
+		for(uint32 j = 0; j < m_vCharset.size(); j++)
 		{
 			nCharsetLen += m_vCharset[j].m_nPlainLenMax;
 			if(i < nCharsetLen) // We found the correct charset
@@ -515,11 +615,28 @@ void CChainWalkContext::IndexToPlain()
 		}
 	}
 
-	UINT4 nIndexOfX32 = (UINT4)nIndexOfX;
+	uint32 nIndexOfX32 = (uint32)nIndexOfX;
 	for (; i >= 0; i--)
 	{
 		int nCharsetLen = 0;
-		for(UINT4 j = 0; j < m_vCharset.size(); j++)
+		for(uint32 j = 0; j < m_vCharset.size(); j++)
+		{
+			nCharsetLen += m_vCharset[j].m_nPlainLenMax;
+			if(i < nCharsetLen) // We found the correct charset
+			{
+
+				m_Plain[i] = m_vCharset[j].m_PlainCharset[nIndexOfX % m_vCharset[j].m_nPlainCharsetLen];
+				nIndexOfX /= m_vCharset[j].m_nPlainCharsetLen;
+				break;
+			}
+		}
+	}
+
+	uint32 nIndexOfX32 = (uint32)nIndexOfX;
+	for (; i >= 0; i--)
+	{
+		int nCharsetLen = 0;
+		for(uint32 j = 0; j < m_vCharset.size(); j++)
 		{
 			nCharsetLen += m_vCharset[j].m_nPlainLenMax;
 			if(i < nCharsetLen) // We found the correct charset
@@ -528,14 +645,12 @@ void CChainWalkContext::IndexToPlain()
 //		m_Plain[i] = m_vCharset[j].m_PlainCharset[nIndexOfX32 % m_vCharset[j].m_nPlainCharsetLen];
 //		nIndexOfX32 /= m_vCharset[j].m_nPlainCharsetLen;
 
-
 //	moving nPlainCharsetLen into the asm body and avoiding the extra temp
 //	variable results in a performance gain
 //				unsigned int nPlainCharsetLen = m_vCharset[j].m_nPlainCharsetLen;
 				unsigned int nTemp;
 
 #if defined(_WIN32) && !defined(__GNUC__)
-
 		// VC++ still needs this
 		unsigned int nPlainCharsetLen = m_vCharset[j].m_nPlainCharsetLen;
 
@@ -549,15 +664,11 @@ void CChainWalkContext::IndexToPlain()
 		}
 		m_Plain[i] = m_vCharset[j].m_PlainCharset[nTemp];
 #else
-		__asm__ __volatile__ (	"mov %2, %%eax;"
-								"xor %%edx, %%edx;"
+		__asm__ __volatile__ ("xor %%edx, %%edx;"
 								"divl %3;"
-								"mov %%eax, %0;"
-								"mov %%edx, %1;"
-								: "=m"(nIndexOfX32), "=m"(nTemp)
-								: "m"(nIndexOfX32), "m"(m_vCharset[j].m_nPlainCharsetLen)
-								: "%eax", "%edx"
-							 );
+								: "=a"(nIndexOfX32), "=d"(nTemp)
+								: "a"(nIndexOfX32), "rm"(m_vCharset[j].m_nPlainCharsetLen)
+								: );
 		m_Plain[i] = m_vCharset[j].m_PlainCharset[nTemp];
 #endif
 		break;
@@ -566,6 +677,7 @@ void CChainWalkContext::IndexToPlain()
 	}
 #endif
 }
+*/
 
 void CChainWalkContext::PlainToHash()
 {	
@@ -586,25 +698,28 @@ const uint64 *CChainWalkContext::GetIndexPtr()
 	return &m_nIndex;
 }
 
-string CChainWalkContext::GetPlain()
+std::string CChainWalkContext::GetPlain()
 {
-	string sRet;
+	std::string sRet;
 	int i;
 	for (i = 0; i < m_nPlainLen; i++)
 	{
 		char c = m_Plain[i];
-		sRet += c;
+		if (c >= 32 && c <= 126)
+			sRet += c;
+		else
+			sRet += '?';
 	}
 	
 	return sRet;
 }
 
-string CChainWalkContext::GetBinary()
+std::string CChainWalkContext::GetBinary()
 {
 	return HexToStr(m_Plain, m_nPlainLen);
 }
 
-string CChainWalkContext::GetHash()
+std::string CChainWalkContext::GetHash()
 {
 	return HexToStr(m_Hash, m_nHashLen);
 }
@@ -617,12 +732,7 @@ bool CChainWalkContext::CheckHash(unsigned char* pHash)
 	return false;
 }
 
-bool CChainWalkContext::isOldFormat()
+uint8 CChainWalkContext::getRTfileFormat()
 {
-	return isOldRtFormat;
-}
-
-bool CChainWalkContext::isRti2Format()
-{
-	return isRti2RtFormat;
+	return RTfileFormatId;
 }
