@@ -6,7 +6,7 @@
  * Copyright Martin Westergaard Jrgensen <martinwj2005@gmail.com>
  * Copyright 2009, 2010  Danil Niggebrugge <niggebrugge@fox-it.com>
  * Copyright 2009 James Dickson
- * Copyright 2009, 2010 James Nobis <frt@quelrod.net>
+ * Copyright 2009, 2010, 2011 James Nobis <quel@quelrod.net>
  * Copyright 2010 uroskn
  *
  * Modified by Martin Westergaard Jrgensen <martinwj2005@gmail.com> to support  * indexed and hybrid tables
@@ -32,10 +32,6 @@
  * along with rcracki_mt.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#if defined(_WIN32) && !defined(__GNUC__)
-	#pragma warning(disable : 4786 4267 4018)
-#endif
-
 #include "CrackEngine.h"
 #include "lm2ntlm.h"
 #include <algorithm>
@@ -43,12 +39,13 @@
 #ifdef _WIN32
 	#include <io.h>
 #else
-	#include <sys/types.h>
-	#include <sys/stat.h>
 	#include <unistd.h>
 	#include <dirent.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #if defined(_WIN32) && !defined(__GNUC__)
 	#pragma comment(lib, "libeay32.lib")
 #endif
@@ -56,12 +53,10 @@
 //////////////////////////////////////////////////////////////////////
 
 #ifdef _WIN32
-void GetTableList(string sWildCharPathName, vector<string>& vPathName)
+void GetTableList( std::string sWildCharPathName, std::vector<std::string>& vPathName )
 {
-	//vPathName.clear();
-
-	string sPath;
-	string::size_type n = sWildCharPathName.find_last_of('\\');
+	std::string sPath;
+	std::string::size_type n = sWildCharPathName.find_last_of('\\');
 
 	if ( n == (sWildCharPathName.size() - 1) )
 	{
@@ -69,7 +64,7 @@ void GetTableList(string sWildCharPathName, vector<string>& vPathName)
 		n = sWildCharPathName.find_last_of('\\');
 	}
 
-	if (n != string::npos)
+	if (n != std::string::npos)
 		sPath = sWildCharPathName.substr(0, n + 1);
 
 	_finddata_t fd;
@@ -79,33 +74,33 @@ void GetTableList(string sWildCharPathName, vector<string>& vPathName)
 	{
 		do
 		{
-			string sName = fd.name;
+			std::string sName = fd.name;
 			if (sName.size()>3) {
 				if (sName.substr(sName.size()-3, 3) == ".rt" && !(fd.attrib & _A_SUBDIR))
 				{
-					string sPathName = sPath + sName;
+					std::string sPathName = sPath + sName;
 					vPathName.push_back(sPathName);
 				}
 			}
 			if (sName.size()>4) {
 				if (sName.substr(sName.size()-4, 4) == ".rti" && !(fd.attrib & _A_SUBDIR))
 				{
-					string sPathName = sPath + sName;
+					std::string sPathName = sPath + sName;
 					vPathName.push_back(sPathName);
 				}
 			}
 			if (sName.size()>5) {
 				if (sName.substr(sName.size()-5, 5) == ".rti2" && !(fd.attrib & _A_SUBDIR))
 				{
-					string sPathName = sPath + sName;
+					std::string sPathName = sPath + sName;
 					vPathName.push_back(sPathName);
 				}
 			}
 
 			if (sName != "." && sName != ".." && (fd.attrib & _A_SUBDIR))
 			{
-				string sPath_sub = sPath + sName + '\\';
-				string sWildCharPathName_sub = sPath_sub + '*';
+				std::string sPath_sub = sPath + sName + '\\';
+				std::string sWildCharPathName_sub = sPath_sub + '*';
 				GetTableList(sWildCharPathName_sub, vPathName);
 			}
 
@@ -117,10 +112,8 @@ void GetTableList(string sWildCharPathName, vector<string>& vPathName)
 }
 #else
 //void GetTableList(int argc, char* argv[], vector<string>& vPathName)
-void GetTableList(string sWildCharPathName, vector<string>& vPathName)
+void GetTableList( std::string sWildCharPathName, std::vector<std::string>& vPathName )
 {
-	//vPathName.clear();
-
 	struct stat buf;
 	if (lstat(sWildCharPathName.c_str(), &buf) == 0)
 	{
@@ -132,11 +125,11 @@ void GetTableList(string sWildCharPathName, vector<string>& vPathName)
 				struct dirent *pDir=NULL;
 				while((pDir = readdir(dir)) != NULL)
 				{
-					string filename = "";
+					std::string filename = "";
 					filename += (*pDir).d_name;
 					if (filename != "." && filename != "..")
 					{
-						string new_filename = sWildCharPathName + '/' + filename;
+						std::string new_filename = sWildCharPathName + '/' + filename;
 						GetTableList(new_filename, vPathName);
 					}
 				}
@@ -173,9 +166,9 @@ void GetTableList(string sWildCharPathName, vector<string>& vPathName)
 }
 #endif
 
-bool NormalizeHash(string& sHash)
+bool NormalizeHash(std::string& sHash)
 {
-	string sNormalizedHash = sHash;
+	std::string sNormalizedHash = sHash;
 
 	if (   sNormalizedHash.size() % 2 != 0
 		|| sNormalizedHash.size() < MIN_HASH_LEN * 2
@@ -183,7 +176,7 @@ bool NormalizeHash(string& sHash)
 		return false;
 
 	// Make lower
-	UINT4 i;
+	uint32 i;
 	for (i = 0; i < sNormalizedHash.size(); i++)
 	{
 		if (sNormalizedHash[i] >= 'A' && sNormalizedHash[i] <= 'F')
@@ -202,20 +195,20 @@ bool NormalizeHash(string& sHash)
 	return true;
 }
 
-void LoadLMHashFromPwdumpFile(string sPathName, vector<string>& vUserName, vector<string>& vLMHash, vector<string>& vNTLMHash)
+void LoadLMHashFromPwdumpFile( std::string sPathName, std::vector<std::string>& vUserName, std::vector<std::string>& vLMHash, std::vector<std::string>& vNTLMHash )
 {
-	vector<string> vLine;
+	std::vector<std::string> vLine;
 	if (ReadLinesFromFile(sPathName, vLine))
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vLine.size(); i++)
 		{
-			vector<string> vPart;
+			std::vector<std::string> vPart;
 			if (SeperateString(vLine[i], "::::", vPart))
 			{
-				string sUserName = vPart[0];
-				string sLMHash   = vPart[2];
-				string sNTLMHash = vPart[3];
+				std::string sUserName = vPart[0];
+				std::string sLMHash   = vPart[2];
+				std::string sNTLMHash = vPart[3];
 
 				if (sLMHash.size() == 32 && sNTLMHash.size() == 32)
 				{
@@ -236,20 +229,20 @@ void LoadLMHashFromPwdumpFile(string sPathName, vector<string>& vUserName, vecto
 }
 
 // 2009-01-04 - james.dickson - Added this so we can load hashes from cain .LST files.
-void LoadLMHashFromCainLSTFile(string sPathName, vector<string>& vUserName, vector<string>& vLMHash, vector<string>& vNTLMHash)
+void LoadLMHashFromCainLSTFile( std::string sPathName, std::vector<std::string>& vUserName, std::vector<std::string>& vLMHash, std::vector<std::string>& vNTLMHash )
 {
-	vector<string> vLine;
+	std::vector<std::string> vLine;
 	if (ReadLinesFromFile(sPathName, vLine))
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vLine.size(); i++)
 		{
-			vector<string> vPart;
+			std::vector<std::string> vPart;
 			if (SeperateString(vLine[i], "\t\t\t\t\t\t", vPart))
 			{
-				string sUserName = vPart[0];
-				string sLMHash   = vPart[4];
-				string sNTLMHash = vPart[5];
+				std::string sUserName = vPart[0];
+				std::string sLMHash   = vPart[4];
+				std::string sNTLMHash = vPart[5];
 
 				if (sLMHash.size() == 32 && sNTLMHash.size() == 32)
 				{
@@ -270,7 +263,7 @@ void LoadLMHashFromCainLSTFile(string sPathName, vector<string>& vUserName, vect
 }
 
 bool NTLMPasswordSeek(unsigned char* pLMPassword, int nLMPasswordLen, int nLMPasswordNext,
-					  unsigned char* pNTLMHash, string& sNTLMPassword)
+					  unsigned char* pNTLMHash, std::string& sNTLMPassword)
 {
 	if (nLMPasswordNext == nLMPasswordLen)
 	{
@@ -304,7 +297,7 @@ bool NTLMPasswordSeek(unsigned char* pLMPassword, int nLMPasswordLen, int nLMPas
 	return false;
 }
 
-bool LMPasswordCorrectCase(string sLMPassword, unsigned char* pNTLMHash, string& sNTLMPassword)
+bool LMPasswordCorrectCase( std::string sLMPassword, unsigned char* pNTLMHash, std::string& sNTLMPassword )
 {
 	if (sLMPassword.size() == 0)
 	{
@@ -313,7 +306,7 @@ bool LMPasswordCorrectCase(string sLMPassword, unsigned char* pNTLMHash, string&
 	}
 
 	unsigned char* pLMPassword = new unsigned char[sLMPassword.size() * 2];
-	UINT4 i;
+	uint32 i;
 	for (i = 0; i < sLMPassword.size(); i++)
 	{
 		pLMPassword[i * 2    ] = sLMPassword[i];
@@ -367,30 +360,30 @@ int main(int argc, char* argv[])
 		return 0;
 	}
 
-	vector<string> vPathName;
-	vector<string> vDefaultRainbowTablePath;
-	string sWildCharPathName			= "";
-	string sInputType						= "";
-	string sInput							= "";
-	string outputFile						= "";
-	string sApplicationPath				= "";
-	string sIniPathName					= "rcracki_mt.ini";
+	std::vector<std::string> vPathName;
+	std::vector<std::string> vDefaultRainbowTablePath;
+	std::string sWildCharPathName			= "";
+	std::string sInputType						= "";
+	std::string sInput							= "";
+	std::string outputFile						= "";
+	std::string sApplicationPath				= "";
+	std::string sIniPathName					= "rcracki_mt.ini";
 	bool writeOutput						= false;
-	string sSessionPathName				= "rcracki.session";
-	string sProgressPathName			= "rcracki.progress";
-	string sPrecalcPathName				= "rcracki.precalc";
+	std::string sSessionPathName				= "rcracki.session";
+	std::string sProgressPathName			= "rcracki.progress";
+	std::string sPrecalcPathName				= "rcracki.precalc";
 	bool resumeSession					= false;
 	bool useDefaultRainbowTablePath	= false;
 	bool debug								= false;
 	bool keepPrecalcFiles				= false;
-	string sAlgorithm						= "";
+	std::string sAlgorithm						= "";
 	int maxThreads							= 1;
 	uint64 maxMem							= 0;
 	CHashSet hs;
 
 	// Read defaults from ini file;
 	bool readFromIni = false;
-	vector<string> vLine;
+	std::vector<std::string> vLine;
 	if (ReadLinesFromFile(sIniPathName, vLine)) {
 		readFromIni = true;
 	}
@@ -399,16 +392,16 @@ int main(int argc, char* argv[])
 	}
 	if (readFromIni)
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vLine.size(); i++)
 		{
 			if (vLine[i].substr(0,1) != "#")
 			{
-				vector<string> vPart;
+				std::vector<std::string> vPart;
 				if (SeperateString(vLine[i], "=", vPart))
 				{
-					string sOption = vPart[0];
-					string sValue  = vPart[1];
+					std::string sOption = vPart[0];
+					std::string sValue  = vPart[1];
 					
 					if (sOption == "Threads") {
 						maxThreads = atoi(sValue.c_str());
@@ -457,7 +450,7 @@ int main(int argc, char* argv[])
 	int i;
 	for (i = 1; i < argc; i++)
 	{
-		string cla = argv[i];
+		std::string cla = argv[i];
 		if (cla == "-h") {
 			sInputType = cla;
 			i++;
@@ -537,17 +530,17 @@ int main(int argc, char* argv[])
 	if (resumeSession)
 	{
 		vPathName.clear();
-		vector<string> sSessionData;
+		std::vector<std::string> sSessionData;
 		if (ReadLinesFromFile(sSessionPathName.c_str(), sSessionData))
 		{
-			UINT4 i;
+			uint32 i;
 			for (i = 0; i < sSessionData.size(); i++)
 			{
-				vector<string> vPart;
+				std::vector<std::string> vPart;
 				if (SeperateString(sSessionData[i], "=", vPart))
 				{
-					string sOption = vPart[0];
-					string sValue  = vPart[1];
+					std::string sOption = vPart[0];
+					std::string sValue  = vPart[1];
 					
 					if (sOption == "sPathName") {
 						vPathName.push_back(sValue);
@@ -581,14 +574,14 @@ int main(int argc, char* argv[])
 	// don't load these if we are resuming a session that already has a list of tables
 	if (useDefaultRainbowTablePath && !resumeSession)
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vDefaultRainbowTablePath.size(); i++)
 		{
-			vector<string> vPart;
+			std::vector<std::string> vPart;
 			if (SeperateString(vDefaultRainbowTablePath[i], ".=", vPart))
 			{
-				string lineAlgorithm = vPart[1];
-				string linePath = vPart[2];
+				std::string lineAlgorithm = vPart[1];
+				std::string linePath = vPart[2];
 
 				if (lineAlgorithm == sAlgorithm)
 					GetTableList(linePath, vPathName);
@@ -608,15 +601,15 @@ int main(int argc, char* argv[])
 		(unsigned long)vPathName.size());
 
 	bool fCrackerType;			// true: hash cracker, false: lm cracker
-	vector<string> vHash;		// hash cracker
-	vector<string> vUserName;	// lm cracker
-	vector<string> vLMHash;		// lm cracker
-	vector<string> vNTLMHash;	// lm cracker
+	std::vector<std::string> vHash;		// hash cracker
+	std::vector<std::string> vUserName;	// lm cracker
+	std::vector<std::string> vLMHash;		// lm cracker
+	std::vector<std::string> vNTLMHash;	// lm cracker
 	if (sInputType == "-h")
 	{
 		fCrackerType = true;
 
-		string sHash = sInput;
+		std::string sHash = sInput;
 		if (NormalizeHash(sHash))
 			vHash.push_back(sHash);
 		else
@@ -626,14 +619,14 @@ int main(int argc, char* argv[])
 	{
 		fCrackerType = true;
 
-		string sPathName = sInput;
-		vector<string> vLine;
+		std::string sPathName = sInput;
+		std::vector<std::string> vLine;
 		if (ReadLinesFromFile(sPathName, vLine))
 		{
-			UINT4 i;
+			uint32 i;
 			for (i = 0; i < vLine.size(); i++)
 			{
-				string sHash = vLine[i];
+				std::string sHash = vLine[i];
 				if (NormalizeHash(sHash))
 					vHash.push_back(sHash);
 				else
@@ -647,14 +640,14 @@ int main(int argc, char* argv[])
 	{
 		fCrackerType = false;
 
-		string sPathName = sInput;
+		std::string sPathName = sInput;
 		LoadLMHashFromPwdumpFile(sPathName, vUserName, vLMHash, vNTLMHash);
 	}
 	else if (sInputType == "-c")
 	{
 		// 2009-01-04 - james.dickson - Added this for cain-files.
 		fCrackerType = false;
-		string sPathName = sInput;
+		std::string sPathName = sInput;
 		LoadLMHashFromCainLSTFile(sPathName, vUserName, vLMHash, vNTLMHash);
 	}
 	else
@@ -676,13 +669,13 @@ int main(int argc, char* argv[])
 
 	if (fCrackerType)
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vHash.size(); i++)
 			hs.AddHash(vHash[i]);
 	}
 	else
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vLMHash.size(); i++)
 		{
 			hs.AddHash(vLMHash[i].substr(0, 16));
@@ -693,25 +686,25 @@ int main(int argc, char* argv[])
 	// Load found hashes from session file
 	if (resumeSession)
 	{
-		vector<string> sSessionData;
+		std::vector<std::string> sSessionData;
 		if (ReadLinesFromFile(sSessionPathName.c_str(), sSessionData))
 		{
-			UINT4 i;
+			uint32 i;
 			for (i = 0; i < sSessionData.size(); i++)
 			{
-				vector<string> vPart;
+				std::vector<std::string> vPart;
 				if (SeperateString(sSessionData[i], "=", vPart))
 				{
-					string sOption = vPart[0];
-					string sValue  = vPart[1];
+					std::string sOption = vPart[0];
+					std::string sValue  = vPart[1];
 					
 					if (sOption == "sHash") {
-						vector<string> vPartHash;
+						std::vector<std::string> vPartHash;
 						if (SeperateString(sValue, "::", vPartHash))
 						{
-							string sHash = vPartHash[0];
-							string sBinary = vPartHash[1];
-							string sPlain = vPartHash[2];
+							std::string sHash = vPartHash[0];
+							std::string sBinary = vPartHash[1];
+							std::string sPlain = vPartHash[2];
 							
 							hs.SetPlain(sHash, sPlain, sBinary);
 						}
@@ -720,19 +713,19 @@ int main(int argc, char* argv[])
 			}
 		}
 	}
-
-	// (Over)write session data if we are not resuming
-	if (!resumeSession)
+	else
 	{
-		FILE* file = fopen(sSessionPathName.c_str(), "w");
-		string buffer = "";
+		FILE* file;
+		
+		std::string buffer = "";
 
-		if (file!=NULL)
+		// (Over)write session data if we are not resuming
+		if ( ( file = fopen( sSessionPathName.c_str(), "w" ) ) != NULL )
 		{
 			buffer += "sInputType=" + sInputType + "\n";
 			buffer += "sInput=" + sInput + "\n";
 
-			UINT4 i;
+			uint32 i;
 			for (i = 0; i < vPathName.size(); i++)
 			{
 				buffer += "sPathName=" + vPathName[i] + "\n";
@@ -747,8 +740,9 @@ int main(int argc, char* argv[])
 			fputs (buffer.c_str(), file);
 			fclose (file);
 		}
-		file = fopen(sProgressPathName.c_str(), "w");
-		fclose (file);
+
+		file = fopen( sProgressPathName.c_str(), "w" );
+		fclose( file );
 	}
 
 	// Run
@@ -775,9 +769,9 @@ int main(int argc, char* argv[])
 	printf("total disk access time:     %.2f s\n", ce.GetStatTotalDiskAccessTime());
 	printf("total cryptanalysis time:   %.2f s\n", ce.GetStatTotalCryptanalysisTime());
 	printf("total pre-calculation time: %.2f s\n", ce.GetStatTotalPrecalculationTime());
-	printf("total chain walk step:      %d\n",     ce.GetStatTotalChainWalkStep());
-	printf("total false alarm:          %d\n",     ce.GetStatTotalFalseAlarm());
-	printf("total chain walk step due to false alarm: %d\n", ce.GetStatTotalChainWalkStepDueToFalseAlarm());
+	printf("total chain walk step:      %llu\n",     ce.GetStatTotalChainWalkStep());
+	printf("total false alarm:          %llu\n",     ce.GetStatTotalFalseAlarm());
+	printf("total chain walk step due to false alarm: %llu\n", ce.GetStatTotalChainWalkStepDueToFalseAlarm());
 //	printf("total chain walk step skipped due to checkpoints: %d\n", ce.GetStatTotalFalseAlarmSkipped()); // Checkpoints not used - yet
 	printf("\n");
 
@@ -786,10 +780,10 @@ int main(int argc, char* argv[])
 	printf("-------------------------------------------------------\n");
 	if (fCrackerType)
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vHash.size(); i++)
 		{
-			string sPlain, sBinary;
+			std::string sPlain, sBinary;
 			if (!hs.GetPlain(vHash[i], sPlain, sBinary))
 			{
 				sPlain  = "<notfound>";
@@ -801,10 +795,10 @@ int main(int argc, char* argv[])
 	}
 	else
 	{
-		UINT4 i;
+		uint32 i;
 		for (i = 0; i < vLMHash.size(); i++)
 		{
-			string sPlain1, sBinary1;
+			std::string sPlain1, sBinary1;
 			bool fPart1Found = hs.GetPlain(vLMHash[i].substr(0, 16), sPlain1, sBinary1);
 			if (!fPart1Found)
 			{
@@ -812,7 +806,7 @@ int main(int argc, char* argv[])
 				sBinary1 = "<notfound>";
 			}
 
-			string sPlain2, sBinary2;
+			std::string sPlain2, sBinary2;
 			bool fPart2Found = hs.GetPlain(vLMHash[i].substr(16, 16), sPlain2, sBinary2);
 			if (!fPart2Found)
 			{
@@ -820,8 +814,8 @@ int main(int argc, char* argv[])
 				sBinary2 = "<notfound>";
 			}
 
-			string sPlain = sPlain1 + sPlain2;
-			string sBinary = sBinary1 + sBinary2;
+			std::string sPlain = sPlain1 + sPlain2;
+			std::string sBinary = sBinary1 + sBinary2;
 
 			// Correct case
 			if (fPart1Found && fPart2Found)
@@ -831,15 +825,21 @@ int main(int argc, char* argv[])
 				ParseHash(vNTLMHash[i], NTLMHash, nHashLen);
 				if (nHashLen != 16)
 					printf("debug: nHashLen mismatch\n");
-				string sNTLMPassword;
+				std::string sNTLMPassword;
 				if (LMPasswordCorrectCase(sPlain, NTLMHash, sNTLMPassword))
 				{
 					sPlain = sNTLMPassword;
 					sBinary = HexToStr((const unsigned char*)sNTLMPassword.c_str(), sNTLMPassword.size());
+					if (writeOutput)
+					{
+						if (!writeResultLineToFile(outputFile, vNTLMHash[i].c_str(), sPlain.c_str(), sBinary.c_str()))
+							printf("Couldn't write final result to file!\n");
+					}
 				}
 				else
 				{
 					printf("%-14s\t%s\thex:%s\n", vUserName[i].c_str(), sPlain.c_str(), sBinary.c_str());
+					printf("Failed case correction, trying unicode correction for: %s\n", sPlain.c_str());
 					LM2NTLMcorrector corrector;
 					if (corrector.LMPasswordCorrectUnicode(sBinary, NTLMHash, sNTLMPassword))
 					{
@@ -852,7 +852,7 @@ int main(int argc, char* argv[])
 						}
 					}
 					else {
-						printf("case correction for password %s failed!\n", sPlain.c_str());
+						printf("unicode correction for password %s failed!\n", sPlain.c_str());
 					}
 				}
 			}
