@@ -3,7 +3,7 @@
  * RainbowCrack
  *
  * Copyright 2009, 2010 Danil Niggebrugge <niggebrugge@fox-it.com>
- * Copyright 2009, 2010 James Nobis <frt@quelrod.net>
+ * Copyright 2009, 2010, 2011 James Nobis <quel@quelrod.net>
  *
  * This file is part of rcracki_mt.
  *
@@ -21,10 +21,6 @@
  * along with rcracki_mt.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#if defined(_WIN32) && !defined(__GNUC__)
-	#pragma warning(disable : 4786 4267 4018)
-#endif
-
 #include "lm2ntlm.h"
 
 LM2NTLMcorrector::LM2NTLMcorrector()
@@ -41,19 +37,19 @@ LM2NTLMcorrector::LM2NTLMcorrector()
 
 }
 
-string LM2NTLMcorrector::getBinary()
+std::string LM2NTLMcorrector::getBinary()
 {
 	return sBinary;
 }
 
-bool LM2NTLMcorrector::LMPasswordCorrectUnicode(string hexPassword, unsigned char* pNTLMHash, string& sNTLMPassword) //, unsigned char* pLMPassword
+bool LM2NTLMcorrector::LMPasswordCorrectUnicode( std::string hexPassword, unsigned char* pNTLMHash, std::string& sNTLMPassword ) //, unsigned char* pLMPassword
 {
-	string sPlain = "";
+	std::string sPlain = "";
 
-	UINT4 i;
+	uint32 i;
 	for (i = 0; i < hexPassword.size() / 2; i++)
 	{
-		string sSub = hexPassword.substr(i * 2, 2);
+		std::string sSub = hexPassword.substr(i * 2, 2);
 		int nValue;
 		sscanf(sSub.c_str(), "%02x", &nValue);
 		sPlain += (unsigned char)nValue;
@@ -66,12 +62,11 @@ bool LM2NTLMcorrector::LMPasswordCorrectUnicode(string hexPassword, unsigned cha
 	unsigned char* pLMPassword = new unsigned char[tmpLength];
 
 	//printf("Searching for unicode password.\n");
-	printf("Failed case correction, trying unicode correction for: %s\n", sPlain.c_str());
 	//printf("NTLM hash:          %s\n\n", sNTLMHash.c_str());
 	
 	setvbuf(stdout, NULL, _IONBF,0);
 
-	startClock = clock();
+	gettimeofday( &tvStart, NULL );
 	previousClock = clock();
 
 #ifndef _WIN32
@@ -100,7 +95,7 @@ bool LM2NTLMcorrector::LMPasswordCorrectUnicode(string hexPassword, unsigned cha
 	}
 }
 
-bool LM2NTLMcorrector::startCorrecting(string sLMPassword, string& sNTLMPassword, unsigned char* pLMPassword)
+bool LM2NTLMcorrector::startCorrecting( std::string sLMPassword, std::string& sNTLMPassword, unsigned char* pLMPassword )
 {
 	if (sLMPassword.size() == 0)
 	{
@@ -108,7 +103,7 @@ bool LM2NTLMcorrector::startCorrecting(string sLMPassword, string& sNTLMPassword
 		return true;
 	}
 
-	string muteMe = sLMPassword;
+	std::string muteMe = sLMPassword;
 	int length = muteMe.size();
 
 	unsigned char* pMuteMe = new unsigned char[length];
@@ -246,7 +241,7 @@ void LM2NTLMcorrector::setupCombinationAtPositions(int length, unsigned char* pM
 }
 
 // go check all permutations for this combination
-bool LM2NTLMcorrector::checkPermutations(int length, unsigned char* pTempMute, int* jAtPos, int* sizeAtPos, unsigned char* pLMPassword, string& sNTLMPassword)
+bool LM2NTLMcorrector::checkPermutations( int length, unsigned char* pTempMute, int* jAtPos, int* sizeAtPos, unsigned char* pLMPassword, std::string& sNTLMPassword )
 {
 	int pos = length - 1;
 
@@ -297,12 +292,16 @@ bool LM2NTLMcorrector::checkPermutations(int length, unsigned char* pTempMute, i
 
 		if (counter > 10000) // don't check clocks too often
 		{
+			gettimeofday( &tvEnd, NULL );
+	
+			tvFinal = sub_timeofday( tvEnd, tvStart );
+
 			clock_t currentClock = clock();
 			float fTime = 1.0f * (currentClock - previousClock);
 			if (fTime > 1.0f * CLOCKS_PER_SEC)
 			{
 				float progressPercentageCurrentCombination = progressCurrentCombination * 100.0f / totalCurrentCombination;
-				float fTime = 1.0f * (currentClock - startClock) / CLOCKS_PER_SEC;
+				float fTime = 1.0f * tvFinal.tv_sec + 1.0f * tvFinal.tv_usec / 1000000;
 				float currentSpeed = (counterOverall + progressCurrentCombination) / fTime / 1000000;
 
 				//printf("%.2f%% of combination %d/%d (%.2f Mhashes/s)\t\t\t\t\r", progressPercentageCurrentCombination, countCombinations, countTotalCombinations, currentSpeed);
@@ -350,17 +349,10 @@ bool LM2NTLMcorrector::checkPermutations(int length, unsigned char* pTempMute, i
 }
 
 // check password, maybe integrate this function in checkPermutations() for performance reasons.
-bool LM2NTLMcorrector::checkNTLMPassword(unsigned char* pLMPassword, int nLMPasswordLen, string& sNTLMPassword)
+bool LM2NTLMcorrector::checkNTLMPassword( unsigned char* pLMPassword, int nLMPasswordLen, std::string& sNTLMPassword )
 {
 	unsigned char md[MD4_DIGEST_LENGTH];
 
-	//MD4(pLMPassword, nLMPasswordLen * 2, md);
-	/*
-	MD4_CTX ctx;
-	MD4_Init(&ctx);
-	MD4_Update(&ctx, pLMPassword, nLMPasswordLen * 2);
-	MD4_Final(md, &ctx);*/ 
-
 	MD4_NEW( pLMPassword, nLMPasswordLen * 2, md );
 
 	if (memcmp(md, NTLMHash, MD4_DIGEST_LENGTH) == 0)
@@ -398,13 +390,13 @@ void LM2NTLMcorrector::checkAbort()
 
 void LM2NTLMcorrector::writeEndStats()
 {
-	clock_t endClock = clock();
-	if (endClock - startClock > 0)
-	{
-		float fTime = 1.0f * (endClock - startClock) / CLOCKS_PER_SEC;
-		float speedOverall = counterOverall / fTime / 1000000;
-		printf("\nTried %s passwords in %.2f s (%.2f Mhashes/s)\n", uint64tostr(counterOverall).c_str(), fTime, speedOverall);
-	}
+	gettimeofday( &tvEnd, NULL );
+	
+	tvFinal = sub_timeofday( tvEnd, tvStart );
+
+	float fTime = 1.0f * tvFinal.tv_sec + 1.0f * tvFinal.tv_usec / 1000000;
+	float speedOverall = counterOverall / fTime / 1000000;
+	printf("\nTried %s passwords in %.2f s (%.2f Mhashes/s)\n", uint64tostr(counterOverall).c_str(), fTime, speedOverall);
 
 	printf("\n");
 }
@@ -414,19 +406,23 @@ int LM2NTLMcorrector::calculateTotalCombinations(int length, int setSize)
 	return factorial(length) / (factorial(setSize) * factorial(length-setSize));
 }
 
-int LM2NTLMcorrector::factorial (int num)
+uint64 LM2NTLMcorrector::factorial (int num)
 {
-	int result = 1;
-	int i;
-	for (i = 1; i <= num; ++i)
-		result *= i;
+	uint64 result = 1;
+
+	while ( num > 1 )
+	{
+		result *= num;
+		num--;
+	}
+
 	return result;
 }
 
 // convert some bytes into a string
-string LM2NTLMcorrector::ByteToStr(const unsigned char* pData, int nLen)
+std::string LM2NTLMcorrector::ByteToStr(const unsigned char* pData, int nLen)
 {
-	string sRet = "";
+	std::string sRet = "";
 	int i;
 	for (i = 0; i < nLen/2; i++)
 	{
